#!/usr/bin/python
import matplotlib
matplotlib.use('Agg')
import matplotlib.cm as cm
from matplotlib.pyplot import figure, savefig
import numpy as np
import sys
import time
import boDb
import logging
import time
import calendar
from boConfig import LOGLEVEL, FTP_LOCAL_PATH, CHANNEL_NUMBER, IMG11_START_DATE_TIME, IMG11_END_DATE_TIME

red = {"dark": "#B70707", "light": "#F00A00"}
green = {"dark": "#3DC20A", "light": "#50FF00"}
yellow = {"dark": "#C1C006", "light": "#FEFE0C"}
magenta = {"dark": "#B905C3", "light": "#F200D6"}
turquoise = {"dark": "#3BA1A3", "light": "#13DEDD"}
orange = {"dark": "#ED6316", "light": "#FF8815"}
gray = {"dark": "#7F7F7F", "light": "#B3B3B3"}
CHANNEL_COLOR = [red, green, yellow, magenta, turquoise, orange, gray]

def generatePolarDiagram(file, data, color, dimension):
    figW = dimension # with of the figure in pixel
    figH = figW # height of the figure in pixel, force square figure and square axes looks better for polar
    fig = figure()
    DPI = fig.get_dpi()
    fig.set_size_inches(figW/float(DPI), figH/float(DPI)) # set size in inches, calculated from pixels and default DPI
    ax = fig.add_axes([0.08, 0.08, 0.84, 0.84], polar=True) # add an axes at position rect [left, bottom, width, height] where all quantities are in fractions of figure width and height
    ax.set_theta_offset(np.pi/2.0) # start north
    ax.set_theta_direction(-1) # count degrees clockwise
    ax.set_ylim(0.0, 105.0) # center at 0%, radius max at 100% with a 5% empty border

    N = len(data)
    width = 2*np.pi/N
    theta = [(2*np.pi*(x+0.5))/N for x in range(N)]
    radii = data

    bars = ax.bar(theta, radii, width=width, bottom=0.0)
    for r, bar in zip(radii, bars):
        bar.set_facecolor(color)
        bar.set_alpha(0.85)

    savefig(file, dpi=DPI) # dpi=DPI sets the same resolution which has been used for the calculations with figW, figH

def getNsFromDateTime(dateTime):
    dateTimeFormat = "%Y.%m.%d %H:%M"
    t = time.strptime(dateTime, dateTimeFormat)
    sec = int(calendar.timegm(t))
    nanoSec = sec * 1000000000
    return nanoSec

def pgStrokesPerChannelVsTotalStrokes():
    dimension = 320 # size in pixel
    N = 360
    
    strokeRatio = {}
    for c in CHANNEL_NUMBER:
        strokeRatio[c] = []

    timeLimit = ""
    if (IMG11_START_DATE_TIME):
        timeLimit += " AND time_ns_stroke > {}".format(getNsFromDateTime(IMG11_START_DATE_TIME))
    if (IMG11_END_DATE_TIME):
        timeLimit += " AND time_ns_stroke <= {}".format(getNsFromDateTime(IMG11_END_DATE_TIME))

    minAzimuth = 0.0
    deltaAzimuth = 360.0 / N
    for i in range(N):
        maxAzimuth = minAzimuth + deltaAzimuth
        SELECT = "count(*) as total"
        for c in CHANNEL_NUMBER:
            channelMask = 1 << c
            SELECT += ", (sum(channels & {}) >> {}) as sum{}".format(channelMask, c, c)
        FROM = " `strokes`"
        WHERE = " channels AND azimuth > {} AND azimuth <= {}".format(minAzimuth, maxAzimuth)
        WHERE += timeLimit
        strokes = boDb.queryTable(SELECT, FROM, WHERE)[0]
        logger.info("minAzimuth = {}, maxAzimuth = {}, strokes = {}".format(minAzimuth, maxAzimuth, strokes))
        for c in CHANNEL_NUMBER:
            if (None != strokes["total"]) and (0 != strokes["total"]) and (None != strokes["sum{}".format(c)]):
                strokeRatio[c].append(strokes["sum{}".format(c)] * 100.0 / strokes["total"]) # calculate percentage
            else:
                strokeRatio[c].append(0) # avoid divide by 0 and calculation with None
        minAzimuth = maxAzimuth
    logger.debug(strokeRatio)

    for c in CHANNEL_NUMBER:
        file = FTP_LOCAL_PATH + '/IMG-i11-c{}.png'.format(c)
        generatePolarDiagram(file, strokeRatio[c], CHANNEL_COLOR[c]["light"], dimension)

def configLogger():
    global logger
    if __name__ == "__main__":
        format = "%(levelname)s: %(message)s"
        logging.basicConfig(format = format)
    logger = logging.getLogger(__name__)
    if __name__ == "__main__":
        # loglevel for standalone usage
        logger.setLevel(logging.INFO)
    else:
        # loglevel when imported
        logger.setLevel(LOGLEVEL)

def main():
    t0 = time.time()

    configLogger()
    boDb.connectMysql()
    boDb.connectDatabase()
    pgStrokesPerChannelVsTotalStrokes()
    boDb.disconnectMysql()

    t1 = time.time()
    logger.info("{} seconds".format(t1-t0))

if __name__ == "__main__":
    main()
